@@ -1,7 +1,6 @@
+'''frontend.py: frontend interface for ufw'''
 #
-# frontend.py: frontend interface for ufw
-#
-# Copyright 2008-2010 Canonical Ltd.
+# Copyright 2008-2012 Canonical Ltd.
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License version 3,
@@ -16,14 +15,13 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import re
 import os
 import sys
 import warnings
 
 from ufw.common import UFWError
 import ufw.util
-from ufw.util import error, warn
+from ufw.util import error, warn, msg
 from ufw.backend_iptables import UFWBackendIptables
 import ufw.parser
 
@@ -54,7 +52,7 @@ def parse_command(argv):
 
     # Show commands
     for i in ['raw', 'before-rules', 'user-rules', 'after-rules', \
-              'logging-rules', 'builtins', 'listening']:
+              'logging-rules', 'builtins', 'listening', 'added']:
         p.register_command(ufw.parser.UFWCommandShow(i))
 
     # Rule commands
@@ -72,25 +70,22 @@ def parse_command(argv):
            argv[idx].lower() in rule_commands:
             argv.insert(idx, 'rule')
 
-    if len(argv) < 2:
-        print >> sys.stderr, "ERROR: not enough args"
-        sys.exit(1)
+    if len(argv) < 2 or ('--dry-run' in argv and len(argv) < 3):
+        error("not enough args")
 
     try:
         pr = p.parse_command(argv[1:])
-    except UFWError, e:
-        print >> sys.stderr, "ERROR: %s" % (e.value)
-        sys.exit(1)
+    except UFWError as e:
+        error("%s" % (e.value))
     except Exception:
-        print >> sys.stderr, "Invalid syntax"
+        error("Invalid syntax", do_exit=False)
         raise
-        sys.exit(1)
 
     return pr
 
 def get_command_help():
     '''Print help message'''
-    msg = _('''
+    help_msg = _('''
 Usage: %(progname)s %(command)s
 
 %(commands)s:
@@ -147,7 +142,7 @@ Usage: %(progname)s %(command)s
          'appupdate': "app update PROFILE", \
          'appdefault': "app default ARG"}))
 
-    return (msg)
+    return (help_msg)
 
 
 class UFWFrontend:
@@ -161,10 +156,7 @@ class UFWFrontend:
         else:
             raise UFWError("Unsupported backend type '%s'" % (backend_type))
 
-        self._init_input_strings()
-
-    def _init_input_strings(self):
-        '''Initialize input strings for translations'''
+        # Initialize input strings for translations
         self.no = _("n")
         self.yes = _("y")
         self.yes_full = _("yes")
@@ -175,28 +167,28 @@ class UFWFrontend:
         '''
         res = ""
 
-        str = "no"
+        config_str = "no"
         if enabled:
-            str = "yes"
+            config_str = "yes"
 
         changed = False
-        if (enabled and not self.backend._is_enabled()) or \
-           (not enabled and self.backend._is_enabled()):
+        if (enabled and not self.backend.is_enabled()) or \
+           (not enabled and self.backend.is_enabled()):
             changed = True
 
         # Update the config files when toggling enable/disable
         if changed:
             try:
                 self.backend.set_default(self.backend.files['conf'], \
-                                         "ENABLED", str)
-            except UFWError, e:
+                                         "ENABLED", config_str)
+            except UFWError as e:
                 error(e.value)
 
         error_str = ""
         if enabled:
             try:
                 self.backend.start_firewall()
-            except UFWError, e:
+            except UFWError as e:
                 if changed:
                     error_str = e.value
 
@@ -206,7 +198,7 @@ class UFWFrontend:
                 try:
                     self.backend.set_default(self.backend.files['conf'], \
                                              "ENABLED", "no")
-                except UFWError, e:
+                except UFWError as e:
                     error(e.value)
 
                 # Report the error
@@ -216,7 +208,7 @@ class UFWFrontend:
         else:
             try:
                 self.backend.stop_firewall()
-            except UFWError, e:
+            except UFWError as e:
                 error(e.value)
 
             res = _("Firewall stopped and disabled on system startup")
@@ -228,10 +220,10 @@ class UFWFrontend:
         res = ""
         try:
             res = self.backend.set_default_policy(policy, direction)
-            if self.backend._is_enabled():
+            if self.backend.is_enabled():
                 self.backend.stop_firewall()
                 self.backend.start_firewall()
-        except UFWError, e:
+        except UFWError as e:
             error(e.value)
 
         return res
@@ -241,7 +233,7 @@ class UFWFrontend:
         res = ""
         try:
             res = self.backend.set_loglevel(level)
-        except UFWError, e:
+        except UFWError as e:
             error(e.value)
 
         return res
@@ -250,16 +242,16 @@ class UFWFrontend:
         '''Shows status of firewall'''
         try:
             out = self.backend.get_status(verbose, show_count)
-        except UFWError, e:
+        except UFWError as e:
             error(e.value)
 
         return out
 
-    def get_show_raw(self, set="raw"):
+    def get_show_raw(self, rules_type="raw"):
         '''Shows raw output of firewall'''
         try:
-            out = self.backend.get_running_raw(set)
-        except UFWError, e:
+            out = self.backend.get_running_raw(rules_type)
+        except UFWError as e:
             error(e.value)
 
         return out
@@ -275,13 +267,13 @@ class UFWFrontend:
 
         rules = self.backend.get_rules()
 
-        protocols = d.keys()
+        protocols = list(d.keys())
         protocols.sort()
         for proto in protocols:
             if not self.backend.use_ipv6() and proto in ['tcp6', 'udp6']:
                 continue
             res += "%s:\n" % (proto)
-            ports = d[proto].keys()
+            ports = list(d[proto].keys())
             ports.sort()
             for port in ports:
                 for item in d[proto][port]:
@@ -327,6 +319,31 @@ class UFWFrontend:
 
         return res
 
+    def get_show_added(self):
+        '''Shows added rules to the firewall'''
+        rules = self.backend.get_rules()
+
+        out = _("Added user rules (see 'ufw status' for running firewall):")
+
+        if len(rules) == 0:
+            return out + _("\n(None)")
+
+        added = []
+        for r in self.backend.get_rules():
+            rstr = ufw.parser.UFWCommandRule.get_command(r)
+
+            # Approximate the order the rules were added. Since rules is
+            # internally rules4 + rules6, IPv6 only rules will show up after
+            # other rules. In terms of rule ordering in the kernel, this is
+            # an equivalent ordering.
+            if rstr in added:
+                continue
+
+            added.append(rstr)
+            out += "\nufw %s" % rstr
+
+        return out
+
     def set_rule(self, rule, ip_version):
         '''Updates firewall with rule'''
         res = ""
@@ -341,12 +358,16 @@ class UFWFrontend:
             try:
                 if rule.remove:
                     if ip_version == "v4":
-                        tmprules = self.backend.get_app_rules_from_system(rule, False)
+                        tmprules = self.backend.get_app_rules_from_system(
+                                                                   rule, False)
                     elif ip_version == "v6":
-                        tmprules = self.backend.get_app_rules_from_system(rule, True)
+                        tmprules = self.backend.get_app_rules_from_system(
+                                                                   rule, True)
                     elif ip_version == "both":
-                        tmprules = self.backend.get_app_rules_from_system(rule, False)
-                        tmprules6 = self.backend.get_app_rules_from_system(rule, True)
+                        tmprules = self.backend.get_app_rules_from_system(
+                                                                   rule, False)
+                        tmprules6 = self.backend.get_app_rules_from_system(
+                                                                   rule, True)
                         # Only add rules that are different by more than v6 (we
                         # will handle 'ip_version == both' specially, below).
                         for x in tmprules:
@@ -467,7 +488,7 @@ class UFWFrontend:
                     else:
                         err_msg = _("Invalid IP version '%s'") % (ip_version)
                         raise UFWError(err_msg)
-            except UFWError, e:
+            except UFWError as e:
                 err_msg = e.value
                 set_error = True
                 break
@@ -486,7 +507,7 @@ class UFWFrontend:
 	    # If error and more than one rule, delete the successfully added
 	    # rules in reverse order
             undo_error = False
-            indexes = range(count+1)
+            indexes = list(range(count+1))
             indexes.reverse()
             for j in indexes:
                 if count > 0 and rules[j]:
@@ -542,7 +563,7 @@ class UFWFrontend:
                        "(%(yes)s|%(no)s)? ") % ({'rule': rstr, \
                                                  'yes': self.yes, \
                                                  'no': self.no})
-            os.write(sys.stdout.fileno(), prompt)
+            msg(prompt, output=sys.stdout, newline=False)
             ans = sys.stdin.readline().lower().strip()
             if ans != "y" and ans != self.yes and ans != self.yes_full:
                 proceed = False
@@ -584,6 +605,8 @@ class UFWFrontend:
             tmp = action.split('-')[1]
             if tmp == "listening":
                 res = self.get_show_listening()
+            elif tmp == "added":
+                res = self.get_show_added()
             else:
                 res = self.get_show_raw(tmp)
         elif action == "status-numbered":
@@ -593,7 +616,7 @@ class UFWFrontend:
         elif action == "disable":
             res = self.set_enabled(False)
         elif action == "reload":
-            if self.backend._is_enabled():
+            if self.backend.is_enabled():
                 self.set_enabled(False)
                 self.set_enabled(True)
                 res = _("Firewall reloaded")
@@ -610,7 +633,7 @@ class UFWFrontend:
                     if tmp != rule.dapp:
                         rule.dapp = tmp
                         rule.set_port(tmp, "dst")
-                except UFWError, e:
+                except UFWError as e:
                     # allow for the profile being deleted (LP: #407810)
                     if not rule.remove:
                         error(e.value)
@@ -624,7 +647,7 @@ class UFWFrontend:
                     if tmp != rule.sapp:
                         rule.sapp = tmp
                         rule.set_port(tmp, "dst")
-                except UFWError, e:
+                except UFWError as e:
                     # allow for the profile being deleted (LP: #407810)
                     if not rule.remove:
                         error(e.value)
@@ -644,14 +667,14 @@ class UFWFrontend:
         res = ""
         try:
             res = self.backend.set_default_application_policy(policy)
-        except UFWError, e:
+        except UFWError as e:
             error(e.value)
 
         return res
 
     def get_application_list(self):
         '''Display list of known application profiles'''
-        names = self.backend.profiles.keys()
+        names = list(self.backend.profiles.keys())
         names.sort()
         rstr = _("Available applications:")
         for n in names:
@@ -662,7 +685,7 @@ class UFWFrontend:
         '''Display information on profile'''
         names = []
         if pname == "all":
-            names = self.backend.profiles.keys()
+            names = list(self.backend.profiles.keys())
             names.sort()
         else:
             if not ufw.applications.valid_profile_name(pname):
@@ -672,7 +695,7 @@ class UFWFrontend:
 
         rstr = ""
         for name in names:
-            if not self.backend.profiles.has_key(name) or \
+            if name not in self.backend.profiles or \
                not self.backend.profiles[name]:
                 err_msg = _("Could not find profile '%s'") % (name)
                 raise UFWError(err_msg)
@@ -721,7 +744,7 @@ class UFWFrontend:
             allow_reload = False
 
         if profile == "all":
-            profiles = self.backend.profiles.keys()
+            profiles = list(self.backend.profiles.keys())
             profiles.sort()
             for p in profiles:
                 (tmp, found) = self.backend.update_app_rule(p)
@@ -735,7 +758,7 @@ class UFWFrontend:
             if rstr != "":
                 rstr += "\n"
 
-        if trigger_reload and self.backend._is_enabled():
+        if trigger_reload and self.backend.is_enabled():
             if allow_reload:
                 try:
                     self.backend._reload_user_rules()
@@ -781,7 +804,7 @@ class UFWFrontend:
         except Exception:
             raise
 
-        if pr.data.has_key('rule'):
+        if 'rule' in pr.data:
             rstr = self.do_action(pr.action, pr.data['rule'], \
                                   pr.data['iptype'])
         else:
@@ -828,7 +851,7 @@ class UFWFrontend:
             prompt = _("Command may disrupt existing ssh connections. " \
                        "Proceed with operation (%(yes)s|%(no)s)? ") % \
                        ({'yes': self.yes, 'no': self.no})
-            os.write(sys.stdout.fileno(), prompt)
+            msg(prompt, output=sys.stdout, newline=False)
             ans = sys.stdin.readline().lower().strip()
             if ans != "y" and ans != self.yes and ans != self.yes_full:
                 proceed = False
@@ -848,13 +871,13 @@ class UFWFrontend:
                        ({'yes': self.yes, 'no': self.no})
 
         if self.backend.do_checks and not force:
-            os.write(sys.stdout.fileno(), ufw.util.wrap_text(prompt))
+            msg(ufw.util.wrap_text(prompt), output=sys.stdout, newline=False)
             ans = sys.stdin.readline().lower().strip()
             if ans != "y" and ans != self.yes and ans != self.yes_full:
                 res = _("Aborted")
                 return res
 
-        if self.backend._is_enabled():
+        if self.backend.is_enabled():
             res += self.set_enabled(False)
         res = self.backend.reset()
 
